package Physics;

import java.awt.Point;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.nio.IntBuffer; //for Java10 com.jogamp.common.nio.Buffers;

//import com.jogamp.common.nio.Buffers;
//import com.jogamp.opengl.GL;
//import com.jogamp.opengl.GLAutoDrawable;
//import com.jogamp.opengl.GLEventListener;
//import com.jogamp.opengl.awt.GLCanvas;
//import com.jogamp.opengl.glu.GLU;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLCanvas; //for Java10 .awt.GLCanvas;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.glu.*;

import com.sun.opengl.util.BufferUtil;

//import cnc.basic.Define;
//import cnc.basic.Vector3d;
//import cnc.cuboid.Cuboid;
//import cnc.cuboid.CuboidCtrlDraw;

//import com.sun.opengl.util.BufferUtil; //for Java10 com.jogamp.opengl.util.*;

/**
 * Simple functions moving, rotating and picking OpenGL-view.
 */
public abstract class BaseGLView extends GLCanvas implements GLEventListener,
  KeyListener, MouseListener, MouseMotionListener, MouseWheelListener
{
	private static final long serialVersionUID = 1L;

	protected GL gl;
	protected GLU glu; //needed for picking.

	// coordination camera
	public double _PositionX = 0.0;
	public double _PositionY = 0.0;
	public double _PositionZ = 0.0;
	
	float _r = .9f;
	public double _tetha = 0.0;
	public double _phi = 0.0;

	boolean _moveView = false;
	boolean _moveObject = false;
	boolean _rotateView = false;
	
	Point _MousePos = new Point();
	Point _downMousePos = new Point();
	Point _oldMousePos = new Point();
	Point nDrag = new Point();
    int nPickingHit = -1; //The PICKING result is saved here. TODO GL.GL_UNSIGNED_INT
    Vector3d vPickingPosition = new Vector3d( Define.UNKNOWN_POSITION, Define.UNKNOWN_POSITION, Define.UNKNOWN_POSITION ); //PICKING: The actual mouse position is hovering over a 3d-point.
	
	public void display(GLAutoDrawable gLDrawable)
	{
		gl = gLDrawable.getGL(); //for Java10 .getGL();
		gl.glClearColor( 0.3f, 0.3f, 0.3f, 0f );
		gl.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
		gl.glDisable( GL.GL_LIGHTING ); //lines do not have a lightning-direction
	    gl.glDisable( GL.GL_DEPTH_TEST ); //sort the faces from back to front
		gl.glDisable( GL.GL_CULL_FACE ); //select only winding-correct

		GLPespective(); //TODO? call here? Needed for zoom-Keys

		gl.glPushMatrix(); //push parameter for the next loop
		gl.glTranslated(_PositionX, _PositionY, _PositionZ);
		gl.glRotated( _tetha, 1.0,  0.0,  0.0 );
		gl.glRotated( _phi, 0.0,  1.0,  0.0 );

		drawCube( 1.0 );
		
		gl.glPopMatrix(); //rotate back for rotation independent working tools like ruler or selectable points
		gl.glFlush();
	}

	/**
	 * Initialize key listener
	 */
	public void init(GLAutoDrawable gLDrawable) {
		glu = new GLU();		
		addKeyListener(this); //gLDrawable.add...
		addMouseListener(this);
		addMouseMotionListener(this);
		addMouseWheelListener(this);		
	}

	/**
	 * Set position of camera.
	 */
	public void GLPespective() { GLPespective( gl, glu, GL.GL_RENDER, getWidth(), getHeight(), _PositionZ, null, null ); }
	public static void GLPespective( GL gl, GLU glu, int mode, double dWidth, double dHeight
									, double _PositionZ, Point point, int viewport[] )
	{
		gl.glMatrixMode( GL.GL_PROJECTION );
		
		if( mode==GL.GL_SELECT )
			gl.glPushMatrix();
		gl.glLoadIdentity();

		if( mode==GL.GL_SELECT )
			glu.gluPickMatrix((double) point.x, (double) (viewport[3] - point.y), 5.0, 5.0, viewport, 0); //GLdouble
		
		if( dWidth > dHeight )
		{
			double aspect = 1.0 * dWidth / dHeight;
			gl.glOrtho( -0.5*_PositionZ*aspect, 0.5*_PositionZ*aspect
					  , -0.5*_PositionZ, 0.5*_PositionZ
					  , -40000.0, 40000.0); //Range -10..10 m
		} else {
			double aspect = 1.0 * dHeight / dWidth;
			gl.glOrtho( -0.5*_PositionZ, 0.5*_PositionZ
					  , -0.5*_PositionZ*aspect, 0.5*_PositionZ*aspect
					  , -40000.0, 40000.0); //Range -10..10 m			
		}
//		Display.getCurrent().getDPI() Parameter -Dsun.java2d.uiScale=1.0 to VM
//		gl.glViewport( 0, 0, (int)dWidth*dScaleFactor, (int)dHeight*dScaleFactor );
		
		gl.glMatrixMode( GL.GL_MODELVIEW );
		if( mode==GL.GL_SELECT )
			gl.glPopMatrix();
		else
			gl.glLoadIdentity();
		gl.glCullFace( GL.GL_FRONT ); //Front is the new standard		

	}

	/**
	 * Returns a value that defines the number of mm per shown pixel
	 * If we have a perspective view it's the value through the coordinate origin.
	 * draw the grid
	 */
	double GetViewSizeFactor()
	{
		if ( getHeight()<=0 || getWidth()<=0 ) return 1.0; //error, window size is zero

		if (getWidth() > getHeight()) return _PositionZ/getHeight();
		else return _PositionZ/getWidth();
	}
	
	public void reshape( GLAutoDrawable gLDrawable, int x, int y
						, int width, int height) {
		gl = gLDrawable.getGL();
		//gl.setSwapInterval(1);
		gl.glViewport( 0, 0, width, height );
//		for Java10 in macos ???? setSize(width, height); setLocation(x, y);		
		GLPespective();
	}	
	public void displayChanged(GLAutoDrawable gLDrawable,
			boolean modeChanged, boolean deviceChanged) {}

	/**
	 * Handle keyboard events
	 * Space-Key: Rotate to a angle modulo 90
	 */
	public void keyTyped(KeyEvent e) {}
	public void keyReleased(KeyEvent e) {}
	public void keyPressed(KeyEvent e) {
		switch( e.getKeyChar() ) {
			case ' ': //perpendicular view 
				_phi = (int)( _phi/90 + Math.signum(_phi)*0.5 ) * 90;
				_tetha = (int)( _tetha/90 + Math.signum(_tetha)*0.5 ) * 90;
				break;
			case 'd': //diagonal view
				_phi = (int)( _phi/90 )*90 + Math.signum(_phi) * 45;
				_tetha = (int)( _tetha/90 )*90 - Math.signum(_phi) * 35.26439;
				break;
			case 's': //half diagonal view
				_phi = (int)( _phi/90 )*90 + Math.signum(_phi) * 45;
				_tetha = (int)( _tetha/90 + Math.signum(_tetha)*0.5 ) * 90;
				break;
		}
	}
	
	/**
	 * Handle mouse
	 * Left mouse button: translate
	 * Middle mouse button: rotate
	 * Right mouse button: pick object
	 * Scroll mouse: Zoom in and out
	 */
	public void mouseClicked( MouseEvent e ) {}
	public void mouseEntered( MouseEvent e ) {}
	public void mouseExited( MouseEvent e ) {}	
	public void mousePressed( MouseEvent e )
	{
		if( e.getButton()==1 ) _rotateView = true;
		if( e.getButton()==2 ) _moveView = true;
		if( e.getButton()==3 ) _moveObject = true; //TODO Button 3 for context menu.

	    display(); //Until we have not redrawn the world we have no PICKING results.
		
		_oldMousePos = e.getPoint(); //Dragging a clamp we need the old mouse-position to get the drag-distance. TODO make a copy of the point?
		_downMousePos = e.getPoint(); 
	}
	public void mouseReleased( MouseEvent e )
	{
		if( e.getButton()==1 ) _rotateView = false;
		if( e.getButton()==2 ) _moveView = false;
		if( e.getButton()==3 ) _moveObject = false; //TODO Button 3 for context menu.
		
	    _oldMousePos = new Point();
	    nDrag = new Point(); //clear the drag way.
	}

	public void mouseMoved( MouseEvent e ) {
		_MousePos = e.getPoint();
	}
	public void mouseDragged( MouseEvent e )
	{
		_MousePos = e.getPoint();
		if( _moveView )
		{
			_PositionX -= (_oldMousePos.x-_MousePos.x ) * GetViewSizeFactor();
			_PositionY += (_oldMousePos.y-_MousePos.y ) * GetViewSizeFactor();
		}

		if( _rotateView )
		{
			//0.1 Degree
			_phi += ( _MousePos.x-_oldMousePos.x ) / 1.0; // % (2 * Math.PI);
			_tetha += ( _MousePos.y-_oldMousePos.y ) / 1.0; // % (2 * Math.PI);
		}

		if( _moveObject )
		{
			nDrag.x += _MousePos.x-_oldMousePos.x; //For picking and dragging clamps.
			nDrag.y += _MousePos.x-_oldMousePos.x; //For picking and dragging clamps.
		}
		
		_oldMousePos = e.getPoint();
		repaint(); //see also FPSAnimator
	}
	
	/**
	 * Zoom in and out relative to the remaining distance of the Profile.
	 */
	public void zoomIn() { _PositionZ -= 0.2*_PositionZ; repaint(); }
	public void zoomOut() { _PositionZ += 0.2*_PositionZ; repaint(); }
    public void mouseWheelMoved( MouseWheelEvent e )
    {
        int notches = e.getWheelRotation();
        _PositionZ += 0.1*notches*_PositionZ;
		repaint(); //see also FPSAnimator
    }

	/**
	 * Draws wire-frame cube. TODO move to another class.
	 * @param  dSize: All sides are this size.
	 */
	public void drawCube( double dSize )
	{ drawCube( 0, 0, 0, dSize,dSize,dSize); }
	public void drawCube( double dMinX, double dMinY, double dMinZ, double dMaxX, double dMaxY, double dMaxZ )
	{
		gl.glColor4d(1.0, 1.0, 1.0, 1.0); //white

		gl.glBegin( GL.GL_LINE_LOOP );
		gl.glVertex3d( dMinX, dMinY, dMinZ );
		gl.glVertex3d( dMaxX, dMinY, dMinZ );
		gl.glVertex3d( dMaxX, dMaxY, dMinZ );
		gl.glVertex3d( dMinX, dMaxY, dMinZ );
		gl.glEnd();

		gl.glBegin( GL.GL_LINE_LOOP );
		gl.glVertex3d( dMinX, dMinY, dMaxZ );
		gl.glVertex3d( dMaxX, dMinY, dMaxZ );
		gl.glVertex3d( dMaxX, dMaxY, dMaxZ );
		gl.glVertex3d( dMinX, dMaxY, dMaxZ );
		gl.glEnd();

		gl.glColor4d(0.7, 0.7, 0.7, 1.0); //white
		
		gl.glBegin( GL.GL_LINES );
		gl.glVertex3d( dMinX, dMinY, dMinZ );
		gl.glVertex3d( dMinX, dMinY, dMaxZ );
		gl.glVertex3d( dMinX, dMaxY, dMinZ );
		gl.glVertex3d( dMinX, dMaxY, dMaxZ );
		gl.glVertex3d( dMaxX, dMinY, dMinZ );
		gl.glVertex3d( dMaxX, dMinY, dMaxZ );
		gl.glVertex3d( dMaxX, dMaxY, dMinZ );
		gl.glVertex3d( dMaxX, dMaxY, dMaxZ );
		gl.glEnd();
	}
	
	/**
	 * Draws wire-frame square. TODO move to another class.
	 * @param dStartX, dStartY, dEndX, dEndY: we can move the corner-points
	 */
	public static void drawSquare( GL gl, double dStartX, double dStartY, double dEndX, double dEndY )
	{
		gl.glDisable( GL.GL_LIGHTING ); //lines do not have a lightning-direction
	
		gl.glBegin( GL.GL_LINE_LOOP );
		gl.glVertex3d( dStartX, dStartY,  0.0 );
		gl.glVertex3d(  dEndX, dStartY,  0.0 );
		gl.glVertex3d(  dEndX,  dEndY,  0.0 );
		gl.glVertex3d( dStartX,  dEndY,  0.0 );
		gl.glEnd();

		gl.glEnable( GL.GL_LIGHTING ); //lines do not have a lightning-direction
	}
	
	/**
	 * PICKING Objects. TODO Problems with near picking objects. see glOrtho()!!!!!
	 * 1. Amount of the names on the stack. 
	 * 2. Smallest z-value of the picked object.
	 * 3. biggest z-value of the picked object.
	 * 4. Name of the object.
	 */
	public abstract void drawPickingObjects( GL gl );
	public int doPickingObjects(GL gl ) throws Exception // Method func  
	{
		// TODO: Add your message handler code here and/or call default
		final int BUFSIZE = 512;
	    int[] selectBuf = new int[BUFSIZE];  //GL.GL_UNSIGNED_INT
	    IntBuffer selectBuffer = BufferUtil.newIntBuffer(BUFSIZE); //for new: Buffers.newDirectIntBuffer(BUFSIZE);

	    int viewport[] = new int[4]; //GL.GL_UNSIGNED_INT
	    	
	    gl.glGetIntegerv( GL.GL_VIEWPORT, viewport, 0 );
	    gl.glSelectBuffer( BUFSIZE, selectBuffer );
		
	    gl.glRenderMode( GL.GL_SELECT );// Enter the SELECT render mode
	    gl.glInitNames();
	    gl.glPushName(-1);

	    GLPespective( gl, glu, GL.GL_SELECT, getWidth(), getHeight(), _PositionZ, _MousePos, viewport ); //_downMousePos
		vPickingPosition.setX( (-_MousePos.x+getWidth()/2.0)*GetViewSizeFactor()+_PositionX );
		vPickingPosition.setY( (_MousePos.y-getHeight()/2.0)*GetViewSizeFactor()+_PositionY );		

		//For picking we use the not rotated or translated object in combination with the mouse-cursor.
		drawPickingObjects( gl ); //drawFunc.call();
		gl.glFlush ();

	    int nHits = gl.glRenderMode ( GL.GL_RENDER );
	    selectBuffer.get(selectBuf);

	    GLPespective();

		if( nHits!=0 ) //see glLoadName( START_INDEX_SELECT_BASICELEMENTS+i ) for machinings
		{
			int n=0;
			int valueZ = selectBuf[1];
			for(int i=1;i<nPickingHit;i++) //Find the nearest object.
			{
				if( selectBuf[1+i*4]<vPickingPosition.getZ() ) { n=i; valueZ = selectBuf[1+i*4]; }
			}
			
			//To scale the click-position divide by 2^32-1 and multiply by view-range see GLPespective()
			double dInvert = (valueZ>0.0) ? -1.0 : 1.0; // System.out.println( valueZ );
			vPickingPosition.setZ( dInvert*valueZ/4294967295.0*80000.0+40000.0 );
			vPickingPosition.setZ( dInvert*vPickingPosition.getZ()+_PositionZ );

			return selectBuf[3+n*4]; //System.out.println( ("Move clamp %nPickingHit% ");
		}
		else
		{
			vPickingPosition.setZ( Define.UNKNOWN_POSITION ); //invalid z-position
			return -1; //Invalidate hits
		}
	}
}
