package Physics;

public class ParticlesSimulation extends Thread
{
	final static int cnMaxGridSizeX = 20; //30 Points
	final static int cnMaxGridSizeY = 20; //40 Points
	int cnGridSizeX = cnMaxGridSizeX; //the used and calculated size
	int cnGridSizeY = cnMaxGridSizeY; //the used and calculated size

	//state of running 
	boolean nStep = false;
	boolean nRun = false;
	boolean nStop = false;
	int nTime = 0;

	double dStep = 0.001;
	double dDistanceFactor = 0.4;
	int ParticleSize = 400;
	final Vector3d aParticle[]; //Position
	final Vector3d aForce[]; //Force of every particle
	final Vector3d aVelocity[]; //Velocity of every particle

	//parameters to configure the simulation. 
	int paramSliderPosition = 0;
	boolean paramCheckContour = false; //display inside and outside-contour
	boolean paramCheckChange = false; //display the material-flow
	boolean paramTemperature = false; //display temperature
	boolean paramCheckFlow = true;
	boolean paramThermalEffects = true; //switch on and off effects of the temperature

	/**
	 * Constructor
	 */
	ParticlesSimulation()
	{
		aParticle = new Vector3d[ ParticleSize ]; //Position
		aForce = new Vector3d[ ParticleSize ]; //Position
		aVelocity = new Vector3d[ ParticleSize ]; //Position
		for( int n=0; n<ParticleSize; n++ )
		{
			aParticle[ n ] = new Vector3d();
			aForce[ n ] = new Vector3d();
			aVelocity[ n ] = new Vector3d();
		}
	}

	/**
	 * Init top layer
	 * Init particles in drop
	 */
	public void Init()
	{
		nTime = 0;
		for( int n=0; n<ParticleSize; n++ )
		{
			aForce[n].setComponents(0.0, 0.0, 0.0);
			aVelocity[n].setComponents(0.0, 0.0, 0.0);
		}

		int Size = 0;
		int radius = 15;
		for( int y=0; y<radius; y++ )
		{
			int xMax = (int) Math.sqrt( radius*radius - y*y );
			for( int x=-xMax; x<xMax; x++ )
//			for( int x=0; x<radius; x++ )
			{
				aParticle[ Size ].setComponents( x,  y, 0 );
				Size++;
			}
		}
		ParticleSize = Size;
	}
	
	/**
	 * Calculate the force between two particles.
	 * f = 20 / r^3 - 8 / r^2
	 */
	public double Force( double radius )
	{
		return 20/(radius*radius*radius) - 8 / (radius*radius*radius*radius*radius);
	}

	//Sum of forces of one particle
	public Vector3d SumForces( int p )
	{
		Vector3d g = new Vector3d( 0.0, 10.0 + nTime/1000, 0.0 );

		Vector3d vForceSum = new Vector3d();
		for( int q=0; q<ParticleSize; q++)
		{
			Vector3d vDistance = aParticle[p].sub( aParticle[q] );
			double dDistance = vDistance.absoluteValue();
			if( dDistance<0.001 ) //same point?
				continue;
			double dForce = Force( dDistance*dDistanceFactor );
			Vector3d vForce = vDistance.mul( dForce ).div( dDistance );
			vForceSum.subEqu( vForce );
		}
		vForceSum = vForceSum.add( g );
		
		return vForceSum;
	}

	/**
	 * Simulation finite particle method
	 */
	public void Step()
	{
		for( int p=0; p<ParticleSize; p++)
			aForce[p] = SumForces( p ); //Force of every particle

			
		for( int p=0; p<ParticleSize; p++)
		{
			if( aParticle[p].getY()<=1 ) //The top layer has to be fixed.
				continue;
			aVelocity[p] = aVelocity[p].add( aForce[p].mul(dStep) );
			aParticle[p] = aParticle[p].add( aVelocity[p].mul(dStep) );
		}
	}
	
	public void SetGridSize( int nSize ) {}
	
	/**
	 * Thread
	 */
	public void run()
	{
		//run step-handling
		while( !nStop )
		{
			if( !nStep && !nRun )
			{
				try {
					sleep(100);
				} catch (InterruptedException e) {
					e.printStackTrace();
				} continue;
			}

			if ( nStep )
				nRun = false; //stop running if step

			nStep = false;
			nTime++;

			Step(); //Generate new inner forces used for material-movements.
			
		} //end while(1)
		
		//nStop=2; //stopped
	}
};
