package Physics;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*; 
import javax.swing.event.*;

//import com.jogamp.opengl.util.FPSAnimator; 
import com.sun.opengl.util.FPSAnimator;

/**
 * Program shows spirals in a 2D and a 3D-space.
 * 
 * The user can make himself a impression about the symmetries 
 * between the 2d- and a 3d-space.
 * Do we live in a 3d space, because the retina in our eye is 2-dimensional?   
 */
public class SpiralMain extends JPanel
{
	private static final long serialVersionUID = 1L;

	static SpiralSimulation _sim = null;
	static FPSAnimator animator = null;
	
	JSplitPane splitPane;

	static SpiralGLView glWindow;
	
	JButton buttonZoomIn;
	JButton buttonZoomOut;
	JSeparator separator;
	JCheckBox buttonColor;
	JCheckBox buttonConstRadius;
	JCheckBox buttonFunction;
	JCheckBox checkBoxMode2D;
	JCheckBox checkBoxSpiral2D;
	JCheckBox checkBoxSpiral3D;
	JCheckBox checkBoxSpiral3D_6d;
	JCheckBox checkBoxSpiralSquare2D;
	JCheckBox checkBoxSpiralSquare3D;
	JCheckBox checkBoxSpiralSquare3D_6d;
	JLabel labelLimit;
	JSlider sliderLimit;
  
	public SpiralMain()
	{
		super( new BorderLayout() );
	
		splitPane = new JSplitPane( 1 ); //vertical split
	     
		_sim = new SpiralSimulation();
	    glWindow = new SpiralGLView( _sim );
	     		
	    //Create the check boxes.
	    buttonZoomIn = new JButton("Zoom in");
	    buttonZoomIn.setMnemonic(KeyEvent.VK_PLUS);
	    buttonZoomOut = new JButton("Zoom out");
	    buttonZoomIn.setMnemonic(KeyEvent.VK_MINUS);
	
	    separator = new JSeparator();
	    buttonColor = new JCheckBox("Color");
	    buttonColor.setMnemonic(KeyEvent.VK_C);
	    buttonColor.setSelected(true);
	    buttonConstRadius = new JCheckBox("Const radius");
	    buttonFunction = new JCheckBox("Functions");
	
	    checkBoxMode2D = new JCheckBox("Mode 2D");
	    checkBoxSpiral2D = new JCheckBox("Spiral plain");
	    checkBoxSpiral3D = new JCheckBox("Spiral");
	    checkBoxSpiral3D_6d = new JCheckBox("Spiral (all)");
	    checkBoxSpiralSquare2D = new JCheckBox("Spiral^2 plain");
	 	checkBoxSpiralSquare3D = new JCheckBox("Spiral^2");
	 	checkBoxSpiralSquare3D_6d = new JCheckBox("Spiral^2 (all)");
	     
	    labelLimit = new JLabel("Limit");
	    sliderLimit = new JSlider( JSlider.HORIZONTAL, 0, 100, _sim.dSliderPos); //FPS_MIN, FPS_MAX, FPS_INIT
	     
	    buttonZoomIn.addActionListener(
	      	new ActionListener() {
	           	public void actionPerformed(ActionEvent e) { glWindow._PositionZ -= 0.2*glWindow._PositionZ; }
	        }
	    );
	    buttonZoomOut.addActionListener(
	       	new ActionListener() {
	           	public void actionPerformed(ActionEvent e) {glWindow._PositionZ += 0.2*glWindow._PositionZ; }
	        }
	    );
	    buttonColor.addActionListener(
	     	new ActionListener() {
	     		public void actionPerformed(ActionEvent e) { _sim.paramCheckColor = !_sim.paramCheckColor; }
	     	}
	    );
	    buttonConstRadius.addActionListener(
		     	new ActionListener() {
		     		public void actionPerformed(ActionEvent e) {
		     			_sim.paramConstRadius = !_sim.paramConstRadius;
		     		}
		     	}
		    );
	    buttonFunction.addActionListener(
		     	new ActionListener() {
		     		public void actionPerformed(ActionEvent e) {
		     			_sim.paramFunction = !_sim.paramFunction;
		     		}
		     	}
		    );
	    
	  	checkBoxMode2D.addActionListener(
	      	new ActionListener() {
	       		public void actionPerformed(ActionEvent e) { _sim.bMode2D = !_sim.bMode2D; }
	       	}
	    );
	
	  	checkBoxSpiral2D.addActionListener(
	       	new ActionListener() {
	       		public void actionPerformed(ActionEvent e) { _sim.bSpiral2D = !_sim.bSpiral2D; }
	       	}
	     );
	  	checkBoxSpiral3D.addActionListener(
	         new ActionListener() {
	         	public void actionPerformed(ActionEvent e) { _sim.bSpiral3D = !_sim.bSpiral3D; }
	         }
	    );
	  	checkBoxSpiral3D_6d.addActionListener(
	  	    new ActionListener() {
	  	       	public void actionPerformed(ActionEvent e) { _sim.bSpiral6S = !_sim.bSpiral6S; }
	  	    }
	  	);
	  	
	  	checkBoxSpiralSquare2D.addActionListener(
	      	new ActionListener() {
	      		public void actionPerformed(ActionEvent e) { _sim.bSpiral2DS = !_sim.bSpiral2DS; }
	      	}
	    );
	  	checkBoxSpiralSquare3D.addActionListener(
	        new ActionListener() {
	        	public void actionPerformed(ActionEvent e) { _sim.bSpiral3DS = !_sim.bSpiral3DS; }
	        }
	    );
	  	checkBoxSpiralSquare3D_6d.addActionListener(
	  		new ActionListener() {
	  	       	public void actionPerformed(ActionEvent e) { _sim.bSpiral6SS = !_sim.bSpiral6SS; }
	  	    }
	  	);
	
	  	sliderLimit.addChangeListener(
	 		new ChangeListener() {
	            	public void stateChanged(ChangeEvent e) { _sim.dSliderPos = sliderLimit.getValue(); }
	         }
	    );
	  	
	    this.add( splitPane );
	     
	    //Put the check boxes in a column in a panel
	    JPanel checkPanel = new JPanel( new GridLayout(0,1) );
	     
	    splitPane.add( checkPanel );
	    splitPane.add( glWindow );
	     
	    checkPanel.add(buttonZoomIn);
	    checkPanel.add(buttonZoomOut);
	    checkPanel.add(buttonColor);
	    checkPanel.add(buttonConstRadius);
	    checkPanel.add(buttonFunction);
	    checkPanel.add(separator);
	     
	    checkPanel.add(checkBoxMode2D);
	    checkPanel.add(checkBoxSpiral2D);
	    checkPanel.add(checkBoxSpiral3D);
	    checkPanel.add(checkBoxSpiral3D_6d);
	    checkPanel.add(checkBoxSpiralSquare2D);
	    checkPanel.add(checkBoxSpiralSquare3D);
	    checkPanel.add(checkBoxSpiralSquare3D_6d);
	
	    checkPanel.add(sliderLimit);
	 }

	 /**
	  * Create the GUI and show it.  For thread safety,
	  * this method should be invoked from the
	  * event-dispatching thread.
	  */
	 private static void createAndShowGUI() {
	     //Create and set up the window.
	     JFrame frame = new JFrame("SpiralDemo");
	     frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	
	     //Create and set up the content pane.
	     JComponent newContentPane = new SpiralMain();
	     newContentPane.setOpaque(true); //content panes must be opaque
	     frame.setContentPane(newContentPane);
	
	     glWindow.addGLEventListener(glWindow);
	
			animator = new FPSAnimator(glWindow, 10); //10 fps
			frame.addWindowListener(new WindowAdapter() {
				//@Override
				public void windowClosing(WindowEvent e) {
					animator.stop();
					System.exit(0);
				}
			});
	     
	    //Display the window.
	    frame.pack();
		frame.setSize(700, 500);
	    frame.setVisible(true);
	    _sim.start();
		animator.start();
	 }
	
	 public static void main(String[] args) {
	     //Schedule a job for the event-dispatching thread:
	     //creating and showing this application's GUI.
	     javax.swing.SwingUtilities.invokeLater(
	     	new Runnable() {
	     		public void run()
	     			{ createAndShowGUI(); }
	     	}
	     );
	 }
}