//import javax.xml.bind.annotation.XmlTransient;
package Physics;

/**
 * 3D-vector-class
 * Add, subtract vectors, scalarProduct, cross-product, rotate vectors
 */
public class Vector3d extends Object
{
	protected double fX;
	protected double fY;
	protected double fZ;

	/**
	 * constructor
	 */
	public Vector3d() { setZero(); }	
	public Vector3d( double ax, double ay, double az ) { setComponents( ax, ay, az ); }
	public Vector3d( Vector3d vect ) { setComponents( vect.fX, vect.fY, vect.fZ ); }
	public Vector3d( double vect[] ) { setComponents( vect[0], vect[1], vect[2] ); }
	public double[] getArray() { return new double[]{ fX, fY, fZ }; }

	/**
	 * get/set see also class VectorFormula
	 */
	//@XmlTransient //XmlAttribute(name = "X", required = true )
	public double getX() { return fX; }
	public void setX( double x ) { fX = x; }
	
	//@XmlTransient //XmlAttribute(name = "Y", required = true )
	public double getY() { return fY; }
	public void setY( double x ) { fY = x; }

	//@XmlTransient //XmlAttribute(name = "Z", required = true )
	public double getZ() { return fZ; }
	public void setZ( double x ) { fZ = x; }
	
	/**
	 * set all components to zero.
	 */
	public void setZero( )
	{
		fX = 0;
		fY = 0;
		fZ = 0;		
	}

	/**
	 * set all components.
	 */
	public void setComponents( double ax, double ay, double az )
	{
		fX = ax;
		fY = ay;
		fZ = az;		
	}

	/**
	 * Copy vector.
	 */
	public void copy( Vector3d vVector )
	{
		fX = vVector.fX;
		fY = vVector.fY;
		fZ = vVector.fZ;
	}
	
	/**
	 * True if all components are 0.
	 */
	public boolean IsNull()
	{
		if( Math.abs(fX)>Define.SAME_POINT )
			return false;
		if( Math.abs(fY)>Define.SAME_POINT )
			return false;
		if( Math.abs(fZ)>Define.SAME_POINT )
			return false;
		return true;
	}
	
	/**
	 * Compares two vectors.
	 */
	public boolean isEqual( Vector3d Vektor )
	{
		if( Math.abs( fX-Vektor.fX )>Define.SAME_POINT )
			return false;
		if( Math.abs( fY-Vektor.fY )>Define.SAME_POINT )
			return false;
		if( Math.abs( fZ-Vektor.fZ )>Define.SAME_POINT )
			return false;
		return true;	
	}
	
	public void addEqu( Vector3d Vektor )
	{
		fX += Vektor.fX;
		fY += Vektor.fY;
		fZ += Vektor.fZ;
	}
	public void addEqu( double x, double y, double z )
	{
		fX += x;
		fY += y;
		fZ += z;
	}
	public void subEqu( Vector3d Vektor )
	{
		fX -= Vektor.fX;
		fY -= Vektor.fY;
		fZ -= Vektor.fZ;
	}
	public void mulEqu( double fFaktor )
	{
		fX *= fFaktor;
		fY *= fFaktor;
		fZ *= fFaktor;
	}
	public void divEqu( double fDivisor )
	{
		fX /= fDivisor;
		fY /= fDivisor;
		fZ /= fDivisor;
	}

	public Vector3d add( Vector3d Vektor )
	{
		return new Vector3d( fX+Vektor.fX, fY+Vektor.fY, fZ+Vektor.fZ );
	}
	public Vector3d sub( Vector3d Vektor )
	{
		return new Vector3d( fX-Vektor.fX, fY-Vektor.fY, fZ-Vektor.fZ );
	}
	public Vector3d mul( double fFaktor )
	{
		return new Vector3d( fX*fFaktor, fY*fFaktor, fZ*fFaktor );
	}
	public Vector3d div( double fDivisor )
	{
		return new Vector3d( fX/fDivisor, fY/fDivisor,fZ/fDivisor );
	}

	/**
	 * Get for each component the smaller/greater value.
	 */
	public Vector3d min( Vector3d Vektor )
	{
		return new Vector3d( Double.min(fX, Vektor.fX)
							,Double.min(fY, Vektor.fY)
							,Double.min(fZ, Vektor.fZ) );
	}
	public Vector3d max( Vector3d Vektor )
	{
		return new Vector3d( Double.max(fX, Vektor.fX)
							,Double.max(fY, Vektor.fY)
							,Double.max(fZ, Vektor.fZ) );
	}
	public void minEqu( Vector3d Vektor )
	{
		fX = Double.min(fX, Vektor.fX);
		fY = Double.min(fY, Vektor.fY);
		fZ = Double.min(fZ, Vektor.fZ);
	}
	public void maxEqu( Vector3d Vektor )
	{
		fX = Double.max(fX, Vektor.fX);
		fY = Double.max(fY, Vektor.fY);
		fZ = Double.max(fZ, Vektor.fZ);
	}
	
	/**
	 * The scalar product has a relation to the angle between the Vectors a*b = |a||b| * cos( a, b ).
	 * The distance between a point p and a plane OQ is ( OQ*n - p*n ) / |n|
	 */
	public double scalarProduct( Vector3d Vektor2 )
	{
		return fX*Vektor2.fX + fY*Vektor2.fY + fZ*Vektor2.fZ; 
	}

	/**
	 * Get the cross-product from this and VektorB.
	 * crossProductEqu can be much faster.
	 */
	public void crossProductEqu( Vector3d VektorB )
	{
		double x = fY*VektorB.fZ - VektorB.fY*fZ;
		double y = fZ*VektorB.fX - VektorB.fZ*fX;
		double z = fX*VektorB.fY - VektorB.fX*fY;
		fX = x; fY = y; fZ = z;
	}
	public Vector3d crossProduct( Vector3d VektorB )
	{
		Vector3d oResult = new Vector3d(
				fY*VektorB.fZ - VektorB.fY*fZ,
				fZ*VektorB.fX - VektorB.fZ*fX,
				fX*VektorB.fY - VektorB.fX*fY
		);
		return oResult;
	}

	public double absoluteValueSquare()
	{
		return fX*fX + fY*fY + fZ*fZ; 
	}
	public double absoluteValue()
	{
		return Math.sqrt(fX*fX + fY*fY + fZ*fZ); 
	}

	/**
	 * calculate the length of the vector.
	 */
	public void normalize()
	{
		double dAbsolute = absoluteValue();
		fX /= dAbsolute;
		fY /= dAbsolute;
		fZ /= dAbsolute;
	}
	
	/**
	 * rotate around x-axis ( right-hand-rule)
	 */
	public Vector3d rotateX( double fAngle )
	{
		fAngle = fAngle % 360.0;
		fAngle *= (Math.PI/180.0);

		double fCos = Math.cos(fAngle);
		double fSin = Math.sin(fAngle);

		double fx = fX;
		double fy = fY*fCos - fZ*fSin;
		double fz = fZ*fCos + fY*fSin;

		setComponents( fx, fy, fz );
		return this;
	}

	/**
	 * rotate around y-axis ( right-hand-rule)
	 */
	public Vector3d rotateY( double fAngle )
	{
	double	fCos, fSin;
	double	fx, fy, fz;

		fAngle = fAngle % 360.0;
		fAngle *= (Math.PI/180.0);

		fCos = Math.cos(fAngle);
		fSin = Math.sin(fAngle);

		fx = fX*fCos + fZ*fSin;
		fy = fY;
		fz = fZ*fCos - fX*fSin;

		setComponents( fx, fy, fz );
		return this;
	}

	/**
	 * rotate around z-axis ( right-hand-rule)
	 */
	public Vector3d rotateZ( double fAngle )
	{
	double	fCos, fSin;
	double	fx, fy, fz;

		fAngle = fAngle % 360.0;
		fAngle *= (Math.PI/180.0);

		fCos = Math.cos(fAngle);
		fSin = Math.sin(fAngle);

		fx = fX*fCos - fY*fSin;
		fy = fY*fCos + fX*fSin;
		fz = fZ;

		setComponents( fx, fy, fz );
		return this;
	}
	
	/**
	 * rotate around x, y and z-axis
	 * TODO: Add a scaling vector for each component.
	 */
	public Vector3d rotateXYZ( double fAngle )
	{		
		fAngle = fAngle % 360.0;
		fAngle *= (Math.PI/180.0); //degree to radiant 
	
		double fCos = Math.cos(fAngle);
		double fSin = Math.sin(fAngle);
	
		//xyz - xzy - yxz - yzx - zxy - zyx

		double fx, fy, fz;
		fx = fX*fCos - fY*fSin + fZ*fSin;
		fy = fY*fCos - fZ*fSin + fX*fSin;
		fz = fZ*fCos - fX*fSin + fY*fSin;

		setComponents( fx, fy, fz );
		return this;
	}

	/**
	 * Rotation angles can be mirrored in direction x-axis by inverting y-z-plane.
	 * Rotation angles can be mirrored in direction y-axis by inverting x-z-plane.
	 */
	public void InvertPlaneYZ() { fY *= -1; fZ *= -1; }
	public void InvertPlaneXZ() { fX *= -1; fZ *= -1; }
	public void InvertPlaneXY() { fX *= -1; fY *= -1; }
	
	// TODO: 4D-rotation with components a1, a2, a3, a4.
	// A rotation does always change two components.
	// Rotate_a1a2_a1a3_a1a4_a2a3_a2a4_a3a4 (around 6 axes)

	/**
	 * Function returns true, if at least one component is zero.
	 */
	public boolean hasZeroComponent()
	{
		if( fX==0.0 || fY==0.0 || fZ==0.0 ) return true;
		return false;
	}

	/**
	 * Perpendicular if two components are zero.
	 */
	public boolean isPerpendicular()
	{
		if( fX==0.0 && fY==0.0 ) return true;
		if( fX==0.0 && fZ==0.0 ) return true;
		if( fY==0.0 && fZ==0.0 ) return true;
		return false;
	}
	
	/************************************************************************************************************/	
	/* Functions where direction-components can be enabled or disabled.                                         */	
	/************************************************************************************************************/	

	/**
	 * If a component is 0 then we get a 1 and if it is 1 we get a 0.
	 */
	public Vector3d invertComponent()
	{
		Vector3d vResult = new Vector3d();
		vResult.fX = 1.0 - fX;
		vResult.fY = 1.0 - fY;
		vResult.fZ = 1.0 - fZ;
		return vResult;
	}
	
	/**
	 * Get the components defined in the vComponent.
	 */
	public Vector3d getComponent( Vector3d vComponent )
	{
		Vector3d vResult = new Vector3d( this );
		vResult.fX *= vComponent.fX;
		vResult.fY *= vComponent.fY;
		vResult.fZ *= vComponent.fZ;
		return vResult;
	}
	
	/**
	 * Rotate vector around x, y and z-axis. Basis is that the rotation is a scaling of two axes.
	 */
	public void rotate( double fAngle, Vector3d vComponent )
	{		
		rotateX( fAngle*vComponent.fX );
		rotateY( fAngle*vComponent.fY );
		rotateZ( fAngle*vComponent.fZ );
	}
	
	/**
	 * Absolute value of a vector if only the components perpendicular to the rotation axis are considered.
	 * TODO: good function?
	 */
	public double absoluteValue( Vector3d vComponent )
	{
		Vector3d vAbsolute = getComponent( vComponent );
		return vAbsolute.absoluteValue();
	}

	/**
	 * Scale some components.
	 */
	public void mulEqu( double factor, Vector3d vComponent )
	{
		fX *= factor*vComponent.fX;
		fY *= factor*vComponent.fY;
		fZ *= factor*vComponent.fZ;
	}

	/**
	 * Add some components.
	 */
	public void addEqu( Vector3d vector, Vector3d vComponent )
	{
		fX += vector.fX*vComponent.fX;
		fY += vector.fY*vComponent.fY;
		fZ += vector.fZ*vComponent.fZ;
	}
	public void subEqu( Vector3d vector, Vector3d vComponent )
	{
		fX -= vector.fX*vComponent.fX;
		fY -= vector.fY*vComponent.fY;
		fZ -= vector.fZ*vComponent.fZ;
	}
	
	/**
	 * Get a value for a component width the right sign.
	 */
	public double getComponentValue( Vector3d vComponent )
	{
		double dResult = 0.0;
		dResult += fX*vComponent.fX;
		dResult += fY*vComponent.fY;
		dResult += fZ*vComponent.fZ;
		return dResult;
	}
}
