package Physics;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*; 
import javax.swing.event.*; 
import com.sun.opengl.util.FPSAnimator;

//import com.jogamp.opengl.util.FPSAnimator;

/**
 * The program should show how a rotation can be interpreted as a symmetry.
 * Also demonstrates waves and how a distortion can rotate the space.
 */
public class WaveMain extends JPanel
{
	private static final long serialVersionUID = 1L;
	
	static WaveSimulation _sim = null;
	static FPSAnimator animator = null;
	
	JSplitPane splitPane;
	
	static WaveGLView glWindow;
	
	JButton buttonZoomIn;
	JButton buttonZoomOut;
	JButton buttonInit;
	JButton buttonRun;
	JButton buttonStep;
	JSeparator separator;
	JCheckBox buttonColor;
	JCheckBox buttonPulse;
	JCheckBox buttonCube;
	JCheckBox buttonRotate;
	JCheckBox buttonFill;    
	JLabel labelSize;
	JSlider sliderSize;
	JLabel labelRotate;
	JSlider sliderRotate;
	 
	public WaveMain()
	{
		super( new BorderLayout() );
		 
		splitPane = new JSplitPane( 1 ); //vertical split
		
		_sim = new WaveSimulation();
		glWindow = new WaveGLView( _sim );
				
		//Create the check boxes.
		buttonZoomIn = new JButton("Zoom in");
		buttonZoomIn.setMnemonic(KeyEvent.VK_PLUS);
		buttonZoomOut = new JButton("Zoom out");
		buttonZoomOut.setMnemonic(KeyEvent.VK_MINUS);
		
		buttonInit = new JButton("Init");
		buttonInit.setMnemonic(KeyEvent.VK_I);
		buttonRun = new JButton("Run");
		buttonRun.setMnemonic(KeyEvent.VK_R);
		buttonStep = new JButton("Step");
		buttonStep.setMnemonic(KeyEvent.VK_S);
		
		separator = new JSeparator(); 
		
		buttonColor = new JCheckBox("Color");
		buttonColor.setMnemonic(KeyEvent.VK_C);
		buttonColor.setSelected(true);
		buttonFill = new JCheckBox("Fill");
		buttonFill.setMnemonic(KeyEvent.VK_A);
		buttonFill.setSelected(false);
		
		buttonPulse = new JCheckBox("Pulse");
		buttonPulse.setMnemonic(KeyEvent.VK_P);
		buttonCube = new JCheckBox("Cube");
		buttonCube.setMnemonic(KeyEvent.VK_Q);
		buttonRotate = new JCheckBox("Rotate");
		buttonRotate.setMnemonic(KeyEvent.VK_R);
		buttonRotate.setSelected(true);

		labelSize = new JLabel("Size");
		sliderSize = new JSlider( JSlider.HORIZONTAL, 0, WaveSimulation.cnMaxGridSize, _sim.cnGridSize); //FPS_MIN, FPS_MAX, FPS_INIT
		
		labelRotate = new JLabel("Rotate");
		sliderRotate = new JSlider( JSlider.HORIZONTAL, 0, 100, _sim.paramSliderAnglePos); //FPS_MIN, FPS_MAX, FPS_INIT
		    
		buttonZoomIn.addActionListener(
				new ActionListener() {
			    	public void actionPerformed(ActionEvent e) { glWindow._PositionZ -= 0.2*glWindow._PositionZ; }
				}
			);
		buttonZoomOut.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {glWindow._PositionZ += 0.2*glWindow._PositionZ; }
				}
	    	);
		buttonInit.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) { _sim.Init(); }
			}
		);
		buttonRun.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) { _sim.bRun=true; }
			}
		);
		buttonStep.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) { _sim.bStep=true; }
			}
		);        
		buttonColor.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) { _sim.paramCheckColor = !_sim.paramCheckColor; }
			}
		);
		buttonFill.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) { _sim.paramCheckRotationFill = !_sim.paramCheckRotationFill; }
			}
		);
		buttonPulse.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) { _sim.paramCheckPulse = !_sim.paramCheckPulse; }
			}
		);
		buttonCube.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) { _sim.paramCheckCubes = !_sim.paramCheckCubes; }
			}
		);
		buttonRotate.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) { _sim.paramCheckRotation = !_sim.paramCheckRotation; }
			}
		);

		sliderSize.addChangeListener(
			new ChangeListener() {
				public void stateChanged(ChangeEvent e) { _sim.SetGridSize(sliderSize.getValue()); }
			}
		);
		sliderRotate.addChangeListener(
			new ChangeListener() {
				public void stateChanged(ChangeEvent e) { _sim.paramSliderAnglePos = sliderRotate.getValue(); }
			}
		);

 
		this.add( splitPane );
		
		//Put the check boxes in a column in a panel
		JPanel checkPanel = new JPanel( new GridLayout(0,1) );
		
		splitPane.add( checkPanel );
		splitPane.add( glWindow );
		
		checkPanel.add(buttonZoomIn);
		checkPanel.add(buttonZoomOut);
		checkPanel.add(buttonInit);
		checkPanel.add(buttonRun);
		checkPanel.add(buttonStep);
		checkPanel.add(separator);
		checkPanel.add(buttonColor);
		checkPanel.add(buttonFill);
		checkPanel.add(separator);
		checkPanel.add(buttonPulse);
		checkPanel.add(buttonCube);
		checkPanel.add(buttonRotate);
		checkPanel.add(labelSize);
		checkPanel.add(sliderSize);
		checkPanel.add(labelRotate);
		checkPanel.add(sliderRotate); //, BorderLayout.SOUTH
	}

    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
    private static void createAndShowGUI() {
        //Create and set up the window.
        JFrame frame = new JFrame("WaveDemo");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
 
        //Create and set up the content pane.
        JComponent newContentPane = new WaveMain();
        newContentPane.setOpaque(true); //content panes must be opaque
        frame.setContentPane(newContentPane);

        glWindow.addGLEventListener(glWindow);

		animator = new FPSAnimator(glWindow, 10); //10 fps
		frame.addWindowListener(new WindowAdapter() {
			//@Override
			public void windowClosing(WindowEvent e) {
				animator.stop();
				System.exit(0);
			}
		});
        
        //Display the window.
        frame.pack();
		frame.setSize(700, 500); 
        frame.setVisible(true);
        _sim.start();
		animator.start();
    }
 
    public static void main(String[] args) {
        //Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(
        	new Runnable() {
        		public void run()
        			{ createAndShowGUI(); }
        	}
        );
    }
}