package Physics;

/**
 * USED IN PhysicsWave
 */
public class WaveSimulation extends Thread
{
	final static double cnGridStiffness = 1.0;
	final static int cnMaxGridSize = 50; //n*n*n Points
	int cnGridSize; //calculated size
	int cnMidPos; //center position

	//state of running 
	boolean bStep = false;
	boolean bRun = false;
	boolean bStop = false;
	int nTime = 0;	
	
	//parameters to configure the simulation. 
	int paramSliderAnglePos = 50;
	boolean paramCheckCubes;
	boolean paramCheckPulse;
	boolean paramCheckRotation = true;
	boolean paramCheckColor = true; //for drawing the grid
	boolean paramCheckRotationFill = false;

	//The energy for X,Y and Z-direction, that is applied to every gridpoint.
	//A negative energy would defines a antiparticle but there are only energy-differences 
	final Vector3d aGridDataMove[][][];

	//The forces in the grid-structure are defining the string-movements.
	final Vector3d aGridDataForce[][][];

	//The speed of every grid-point ( -1 for negative speed and 1 for positive speed )
	final Vector3d aGridDataSpeed[][][];
	
	/**
	 * Orthogonal linking between points.
	 */
	final static int nConnects=6;
	final static VectorInt oConnect[] = {
		new VectorInt( 1, 0, 0 ),
		new VectorInt(-1, 0, 0 ),
		new VectorInt( 0, 1, 0 ),
		new VectorInt( 0,-1, 0 ),
		new VectorInt( 0, 0, 1 ),
		new VectorInt( 0, 0,-1 )
	};


	/**
	 * Constructor
	 */
	WaveSimulation()
	{
//		pThread=NULL;
		aGridDataMove = new Vector3d[cnMaxGridSize][cnMaxGridSize][cnMaxGridSize];
		aGridDataForce = new Vector3d[cnMaxGridSize][cnMaxGridSize][cnMaxGridSize];
		aGridDataSpeed = new Vector3d[cnMaxGridSize][cnMaxGridSize][cnMaxGridSize];
		
		for( int z=0; z<cnMaxGridSize; z++)
			for( int y=0; y<cnMaxGridSize; y++)
				for( int x=0; x<cnMaxGridSize; x++)
				{
					aGridDataMove[z][y][x] = new Vector3d();
					aGridDataSpeed[z][y][x] = new Vector3d();
					aGridDataForce[z][y][x] = new Vector3d();
				}

		SetGridSize( 25 );
	}

	/**
	 * scale grid
	 */
	void SetGridSize( int nSize )
	{
		cnGridSize = nSize; //calculated size
		cnMidPos = cnGridSize/2; //center position
	}
	
	/**
	 * A plane grid has no energy!
	 * A gravity-force is created by bend the space into a spherical structure.
	 */
	void InitRotation() 
	{
		for( int z=0; z<cnGridSize; z++)
			for( int y=0; y<cnGridSize; y++)
				for( int x=0; x<cnGridSize; x++)
				{
					if( !paramCheckRotationFill ) //only cube contour?
					{
						if( !( (x==1 && y==1)
							|| (x==1 && z==1)
							|| (y==1 && z==1)
							|| (x==cnGridSize-2 && y==1)
							|| (x==cnGridSize-2 && z==1)
							|| (x==1 && y==cnGridSize-2)
							|| (y==cnGridSize-2 && z==1)
							|| (x==1 && z==cnGridSize-2)
							|| (y==1 && z==cnGridSize-2) 
							|| (x==cnGridSize-2 && y==cnGridSize-2)
							|| (x==cnGridSize-2 && z==cnGridSize-2)
							|| (y==cnGridSize-2 && z==cnGridSize-2)	) )
							continue;
					}

//					double distSquare = (x-cnMidPos)*(x-cnMidPos)
//									  + (y-cnMidPos)*(y-cnMidPos)
//									  + (z-cnMidPos)*(z-cnMidPos);
//					double dist = (x-cnMidPos)
//								+ (y-cnMidPos)
//								+ (z-cnMidPos);

					aGridDataMove[z][y][x].setComponents( 0.3*(x-cnMidPos)
														, 0.3*(y-cnMidPos)
														, 0.3*(z-cnMidPos) );

					
					//8 corners and 8 possibilities to bend the grid
					//{1,1,1}, {-1,1,1}, {1,-1,1}, {-1,-1,1},
					//{1,1,-1}, {-1,1,-1}, {1,-1,-1}, {-1,-1,-1},
					
					for( int i=0; i<paramSliderAnglePos; i++ ) //dist
					{
						//For all possible states alter to negative angle and mirror coordinate system.
						aGridDataMove[z][y][x].rotateXYZ( 1 );
						//aGridDataMove[z][y][x].RotateX( 1 );
						//aGridDataMove[z][y][x].RotateY( 1 );
						//aGridDataMove[z][y][x].RotateZ( 1 );
					}
				}
	}

	/**
	 * Initialize a pulse wave
	 */
	void InitPulse() 
	{		
		if( !paramCheckRotationFill )
		{
			//Pulse
			double dDisortion=3;
			aGridDataMove[cnMidPos-1][cnMidPos-1][cnMidPos-1].setComponents( -dDisortion, -dDisortion, -dDisortion );
			aGridDataMove[cnMidPos-1][cnMidPos-1][cnMidPos+1].setComponents( dDisortion, -dDisortion, -dDisortion );
			aGridDataMove[cnMidPos-1][cnMidPos-1][cnMidPos-1].setComponents( -dDisortion, -dDisortion, -dDisortion );
			aGridDataMove[cnMidPos-1][cnMidPos+1][cnMidPos-1].setComponents( -dDisortion, dDisortion, -dDisortion );
			aGridDataMove[cnMidPos-1][cnMidPos+1][cnMidPos+1].setComponents( dDisortion, dDisortion, -dDisortion );

			aGridDataMove[cnMidPos+1][cnMidPos-1][cnMidPos-1].setComponents( -dDisortion, -dDisortion, dDisortion );
			aGridDataMove[cnMidPos+1][cnMidPos-1][cnMidPos+1].setComponents( dDisortion, -dDisortion, dDisortion );
			aGridDataMove[cnMidPos+1][cnMidPos-1][cnMidPos-1].setComponents( -dDisortion, -dDisortion, dDisortion );
			aGridDataMove[cnMidPos+1][cnMidPos+1][cnMidPos-1].setComponents( -dDisortion, dDisortion, dDisortion );
			aGridDataMove[cnMidPos+1][cnMidPos+1][cnMidPos+1].setComponents( dDisortion, dDisortion, dDisortion );			

			//double dDisortion=5;
			//aGridDataMove[cnMidPos][cnMidPos][cnMidPos-1].getX() = -dDisortion;
			//aGridDataMove[cnMidPos][cnMidPos][cnMidPos+1].getX() = dDisortion;
			//aGridDataMove[cnMidPos][cnMidPos-1][cnMidPos].getY() = -dDisortion;
			//aGridDataMove[cnMidPos][cnMidPos+1][cnMidPos].getY() = dDisortion;
			//aGridDataMove[cnMidPos-1][cnMidPos][cnMidPos].getZ() = -dDisortion;
			//aGridDataMove[cnMidPos+1][cnMidPos][cnMidPos].getZ() = dDisortion;
		}
		else
		{
			//Pulse filled 
			double dFactorPotential = 3.0;
			Vector3d oCenter = new Vector3d( 0.5*cnGridSize, 0.5*cnGridSize, 0.5*cnGridSize );
			Vector3d oDist = new Vector3d();
	
			for( int z=1; z<cnGridSize-1; z++) {
				for( int y=1; y<cnGridSize-1; y++) {
					for( int x=1; x<cnGridSize-1; x++) {
						oDist.setComponents(x,y,z);
						oDist.subEqu( oCenter );
	
						double nDistanceSquare = oDist.absoluteValueSquare();
						//ASSERT( nDistanceSquare!=0 );
						aGridDataMove[z][y][x] = oDist.mul(-dFactorPotential / nDistanceSquare);
					}
				}
			}
		}
	}

	/**
	 * Initialize fields with different directions.
	 */
	void InitCubes() 
	{
		//dextrorotatory field (rechtsdrehend) 
		final double cdA = 1;
		final double cdB = 0.3;
		final double cdV = 0.8;
		final double cdW = Math.sqrt( 3.0 );
		//Vector3d oCenter = new Vector3d( 0.5*cnGridSize, 0.5*cnGridSize, 0.5*cnGridSize );

		for( int z=1; z<cnGridSize-1; z++) {
			for( int y=1; y<cnGridSize-1; y++) {
				for( int x=1; x<cnGridSize-1; x++) {
					if( x>0.5*cnGridSize && y>0.5*cnGridSize && z<0.5*cnGridSize ) aGridDataMove[z][y][x].setComponents(  cdA,  cdV,  -cdB); //I
					if( x<0.5*cnGridSize && y>0.5*cnGridSize && z<0.5*cnGridSize ) aGridDataMove[z][y][x].setComponents( -cdW,  cdB,  -cdV); //II
					if( x<0.5*cnGridSize && y<0.5*cnGridSize && z<0.5*cnGridSize ) aGridDataMove[z][y][x].setComponents( -cdA, -cdV,  -cdB); //III
					if( x>0.5*cnGridSize && y<0.5*cnGridSize && z<0.5*cnGridSize ) aGridDataMove[z][y][x].setComponents(  cdV, -cdB,  -cdW); //IV

					if( x>0.5*cnGridSize && y>0.5*cnGridSize && z>0.5*cnGridSize ) aGridDataMove[z][y][x].setComponents(  cdW,  cdA,  cdV); //I
					if( x<0.5*cnGridSize && y>0.5*cnGridSize && z>0.5*cnGridSize ) aGridDataMove[z][y][x].setComponents( -cdA,  cdV,  cdB); //II
					if( x<0.5*cnGridSize && y<0.5*cnGridSize && z>0.5*cnGridSize ) aGridDataMove[z][y][x].setComponents( -cdV, -cdA,  cdW); //III
					if( x>0.5*cnGridSize && y<0.5*cnGridSize && z>0.5*cnGridSize ) aGridDataMove[z][y][x].setComponents(  cdA, -cdV,  cdB); //IV
				}
			}
		}
	}

	/**
	 * Initialize start-conditions
	 */
	void Init() 
	{
		//Initialize COMPLETE grid with zero-values! 
		for( int z=0; z<cnMaxGridSize; z++) {
			for( int y=0; y<cnMaxGridSize; y++) {
				for( int x=0; x<cnMaxGridSize; x++) {
					aGridDataMove[z][y][x].setZero();
					aGridDataSpeed[z][y][x].setZero();
					aGridDataForce[z][y][x].setZero();
				}
			}
		}

		//generate source
		if( paramCheckCubes ) InitCubes();
		if( paramCheckRotation ) InitRotation();
		if( paramCheckPulse ) InitPulse();
	}

	/**
	 * Calculate a forces between two points
	 * We assume a linear force between the points as we assume there is a spring placed between the points.
	 *
	 * Return ForceVector
	 */
	Vector3d AddForceComponent( Vector3d oPosVector, Vector3d oConnectVector,
								Vector3d oMoveVector, Vector3d oMoveNextVector )
	{
		Vector3d oDistance = oMoveNextVector.add(oConnectVector).sub(oMoveVector); //neighbor position - actual Position
		double dDistance = oDistance.absoluteValue();
		if( dDistance<0 ) System.out.print("dDistance>0\n");
		if( dDistance>1000 ) System.out.print("dDistance<1000\n");

		//The spring-length is the distance between the 2 points
		double dSpringLength = dDistance*dDistance;

		// Force is square to distance. 
		// ForceX = spring-force * x-component
		// spring-force = (cdK*dSpringLength + cdL) 
		// x-component = nDistX/nDistance

		double cdK = 0.0007; //spring constant * distance between two points = force
		double cdL = -0.0005; //-0.00005 the initial force in a spring in a not disortet grid (initial tension of our universe).

		return oDistance.mul( ((cdK*dSpringLength + cdL) / dDistance) );
	}


	/**
	 * Calculate the forces that apply to every point to the grid.
	 * The force is simply the 
	 * The forces in the grid-structure are defining the string-movements.	
	 */
	void CalcForce( int px, int py, int pz )
	{
		aGridDataForce[pz][py][px].setZero(); //set null
		Vector3d oPosVector = new Vector3d( px, py, pz); //actual position

		//START sphere-evaluation
//		Vector3d oCenterVector = new Vector3d( (0.5*cnGridSize-0.5 ), (0.5*cnGridSize-0.5), (0.5*cnGridSize-0.5) );
//		if( oPosVector.sub(oCenterVector).AbsoluteValueSquare()>(0.5*cnGridSize-0.5 )*(0.5*cnGridSize-0.5 ) )
//			return;
		//END sphere-evaluation

		Vector3d dForceVector = new Vector3d();
		for( int d=0; d<nConnects; d++ )
		{
			// Calculate distance to the neighbor points
			Vector3d oConnectVector = new Vector3d( oConnect[d].getX(), oConnect[d].getY(), oConnect[d].getZ());
			Vector3d oMoveNextVector = new Vector3d(
					aGridDataMove[ pz+oConnect[d].getZ() ][ py+oConnect[d].getY() ][ px+oConnect[d].getX() ].getX(),
					aGridDataMove[ pz+oConnect[d].getZ() ][ py+oConnect[d].getY() ][ px+oConnect[d].getX() ].getY(),
					aGridDataMove[ pz+oConnect[d].getZ() ][ py+oConnect[d].getY() ][ px+oConnect[d].getX() ].getZ() 
			);

			dForceVector = AddForceComponent( oPosVector, oConnectVector, aGridDataMove[pz][py][px], oMoveNextVector );
			aGridDataForce[pz][py][px].addEqu( dForceVector ); // 6.0; //assign Forces
		}
	}


	/**
	 * Calculate the forces that apply to every point to the grid.
	 * The force is simply the 
	 * The forces in the grid-structure are defining the string-movements.	
	 *
	 * Use Laplace-operator in 3D
	 *	      1
	 *	      | 1
	 * 1 - -6  - 1
	 *	    1 |
	 *	      1
	 */
	void CalcMove( int px, int py, int pz )
	{
		int d;
		for( d=0; d<nConnects; d++ ) // Calculate accelerations
			aGridDataSpeed[pz][py][px].addEqu( aGridDataForce[pz][py][px] );

		//damping oscillation! 
		double cdC = 0.0; //damping coefficient c
		aGridDataSpeed[pz][py][px].subEqu(  aGridDataSpeed[pz][py][px].mul( cdC )  );

		for( d=0; d<nConnects; d++ ) // Calculate movements
			aGridDataMove[pz][py][px].addEqu( aGridDataSpeed[pz][py][px] );
	}

	
	/**
	 * Thread
	 */
	public void run()
	{
		//run step-handling
		while( !bStop )
		{
			if( !bStep && !bRun )
			{
				try {
					sleep(100);
				} catch (InterruptedException e) {
					e.printStackTrace();
				} continue;
			}

			if ( bStep )
				bRun=false; //stop running if step

			bStep=false;
			nTime++;

			//generate the forces inside our grid
			for( int z=1; z<cnGridSize-1; z++) {
			  for( int y=1; y<cnGridSize-1; y++) {
				for( int x=1; x<cnGridSize-1; x++) {
					CalcForce(x,y,z);
				}
			  }
			}

			//generate movements of our wave the
			for( int z=1; z<cnGridSize-1; z++) {
				for( int y=1; y<cnGridSize-1; y++) {
					for( int x=1; x<cnGridSize-1; x++) {
						CalcMove(x,y,z);
					}
				}
			}
		} //end while(1)
		
		//nStop=2; //stopped
	}
};
